/** 
* Copyright (c) 2007 Guido Angenendt <guido.angenendt@stud.uni-due.de>
*
*    This file is part of hellaKommanda.
*
*    hellaKommanda is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    hellaKommanda is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with hellaKommanda.  If not, see <http://www.gnu.org/licenses/>.
**/

//
// Global variables
//
var loop;
var serverUrl;
var priXmlRpcClient;

//
// Functions
//

// Start loop on getStatus() function
function main() {
	serverUrl = getUrl();
	if (serverUrl) {
		loop = window.setInterval("getStatus();", 1000);
	}
}

// Create instace of xmlrpc client
function getClient() {

	// Grant rights
	netscape.security.PrivilegeManager.enablePrivilege(
		'UniversalXPConnect UniversalBrowserAccess');
	
	// Return instance	
	return Components.classes['@mozilla.org/xml-rpc/client;1'].createInstance(
		Components.interfaces.nsIXmlRpcClient);
}

// Get instance of (primary) xmlrpc client
function getPriXmlRpc() {

	if (!priXmlRpcClient) {
		priXmlRpcClient = getClient();
		}
		
	if (!serverUrl) {
		serverUrl = getUrl();
		}
	
	priXmlRpcClient.init(serverUrl);

	return priXmlRpcClient;
}

// Get server url from settings file
function getUrl() {
	
	//load xml file
	var xmlDoc = document.implementation.createDocument("","",null);
	xmlDoc.async = false;
	
	try { xmlDoc.load("chrome://hellakommanda/content/settings.xml");}
	catch (e) {}
	
	// Read values
	var server = xmlDoc.getElementsByTagName("server").item(0).firstChild.nodeValue;
	var port = xmlDoc.getElementsByTagName("port").item(0).firstChild.nodeValue;
	var password = xmlDoc.getElementsByTagName("password").item(0).firstChild.nodeValue;
	
	return "http://hellanzb:" + password + "@" + server + ":" + port;
}

// Ask for status
function getStatus() {
	
	// Grant rights
	netscape.security.PrivilegeManager.enablePrivilege(
		'UniversalXPConnect UniversalBrowserAccess');
	// Get the instance of the XML-RPC client
	var xmlRpcClient = getPriXmlRpc();
	// Send call
	xmlRpcClient.asyncCall(getStatusHandler, null, 'status', [], 0);
}

// Status Handler
// Gets status values and creates new datasource
var getStatusHandler = {

	onResult: function(client, ctxt, result) {
		
		// Grant rights
		netscape.security.PrivilegeManager.enablePrivilege(
			'UniversalXPConnect UniversalBrowserAccess');
		
		// Create components
		var rdf = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService();
		rdf = rdf.QueryInterface(Components.interfaces.nsIRDFService);
		var ds = Components
			.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
			.createInstance();
		ds = ds.QueryInterface(Components.interfaces.nsIRDFDataSource);

		// Create root node
		var rootNode =  rdf.GetResource("urn:root");
		
		// Create three resources that are
        // children of urn:root
		var currentNode =  rdf.GetResource("urn:current");
		var queueNode = rdf.GetResource("urn:queue");
		var logNode = rdf.GetResource("urn:log");
		
		//Append children
		ds.Assert(rootNode,
			rdf.GetResource("http://home.netscape.com/NC-rdf#current"),
			currentNode, true);
		ds.Assert(rootNode,
			rdf.GetResource("http://home.netscape.com/NC-rdf#queue"),
			queueNode, true);
		ds.Assert(rootNode,
			rdf.GetResource("http://home.netscape.com/NC-rdf#log"),
			logNode, true);

		// Get trees and add datasource
        for (var i = 1; i <= 3; ++i) {
			var tree = document.getElementById("tree" + i);
			var enm = tree.database.GetDataSources();
			while (enm.hasMoreElements()){
				var datasource = enm.getNext();
				tree.database.RemoveDataSource(datasource);
				}
			tree.database.AddDataSource(ds);
        }
        
        // Get status
		var statusResult = result
						.QueryInterface(Components.interfaces.nsIDictionary);
						
		// Prepare current data
		var cur_arr = statusResult.getValue('currently_downloading')
						.QueryInterface(Components.interfaces.nsISupportsArray);
		var percent = statusResult.getValue('percent_complete')
						.QueryInterface(Components.interfaces.nsISupportsPRInt32)
						.toString() + " %";
		var rate = statusResult.getValue('rate')
						.QueryInterface(Components.interfaces.nsISupportsPRInt32);
		var eta = statusResult.getValue('eta')
						.QueryInterface(Components.interfaces.nsISupportsPRInt32);

		// Format eta
		if(eta.data > 3600) {
			eta.data = Math.round(eta.data / 3600);
			eta = eta.toString() + " h";
			}
		else if(eta.data > 60) {
			eta.data = Math.round(eta.data / 60);
			eta = eta.toString() + " min";
			}
		else {
			eta = eta.toString() + " s";
		}
			
		if (cur_arr.Count() > 0) {
			var cur_dic = cur_arr
						.QueryElementAt(0, Components.interfaces.nsIDictionary);
			var nzbName = cur_dic.getValue('nzbName')
						.QueryInterface(Components.interfaces.nsISupportsCString);
			var total_mb = cur_dic.getValue('total_mb')
						.QueryInterface(Components.interfaces.nsISupportsPRInt32);

			// Add current data
			// nzbName
			ds.Assert(rdf.GetResource("urn:current.0"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#currentName"),
				rdf.GetLiteral(nzbName), true);
			ds.Assert(rdf.GetResource("urn:current"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#current"),
				rdf.GetResource("urn:current.0"), true);
			// total_mb
			ds.Assert(rdf.GetResource("urn:current.0"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#currentMb"),
				rdf.GetLiteral(total_mb), true);
			ds.Assert(rdf.GetResource("urn:current"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#current"),
				rdf.GetResource("urn:current.0"), true);
			// percent
			ds.Assert(rdf.GetResource("urn:current.0"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#currentPercent"),
				rdf.GetLiteral(percent), true);
			ds.Assert(rdf.GetResource("urn:current"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#current"),
				rdf.GetResource("urn:current.0"), true);
			// rate
			ds.Assert(rdf.GetResource("urn:current.0"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#currentRate"),
				rdf.GetLiteral(rate), true);
			ds.Assert(rdf.GetResource("urn:current"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#current"),
				rdf.GetResource("urn:current.0"), true);
			// eta
			ds.Assert(rdf.GetResource("urn:current.0"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#currentEta"),
				rdf.GetLiteral(eta), true);
			ds.Assert(rdf.GetResource("urn:current"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#current"),
				rdf.GetResource("urn:current.0"), true);
		}
		
		// Prepare queue data
		var queue_arr = statusResult.getValue('queued')
					.QueryInterface(Components.interfaces.nsISupportsArray);
		if(queue_arr.Count() > 0) {
		for (var i = 0; i < queue_arr.Count(); i++) {
			var newURI = "urn:queue." + i;
			var queue_dic = queue_arr.QueryElementAt(i, Components.interfaces.nsIDictionary);
			var id = queue_dic.getValue('id')
						.QueryInterface(Components.interfaces.nsISupportsPRInt32);
			var nzbName = queue_dic.getValue('nzbName')
							.QueryInterface(Components.interfaces.nsISupportsCString);
							
			if(queue_dic.hasKey('total_mb')) {
				var total_mb = queue_dic.getValue('total_mb')
						.QueryInterface(Components.interfaces.nsISupportsPRInt32);
			} else {
				var total_mb = "?";
			}
			
	 		// Add queue data
	 		// id
			ds.Assert(rdf.GetResource(newURI),
				rdf.GetResource("http://home.netscape.com/NC-rdf#queueId"),
				rdf.GetLiteral(id), true);
			ds.Assert(rdf.GetResource("urn:queue"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#queue"),
				rdf.GetResource(newURI), true);
		 	// nzbName
			ds.Assert(rdf.GetResource(newURI),
				rdf.GetResource("http://home.netscape.com/NC-rdf#queueName"),
				rdf.GetLiteral(nzbName), true);
			ds.Assert(rdf.GetResource("urn:queue"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#queue"),
				rdf.GetResource(newURI), true);
	 		// total_mb
			ds.Assert(rdf.GetResource(newURI),
				rdf.GetResource("http://home.netscape.com/NC-rdf#queueMb"),
				rdf.GetLiteral(total_mb), true);
			ds.Assert(rdf.GetResource("urn:queue"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#queue"),
				rdf.GetResource(newURI), true);
			}
		}

		// Prepare log data		
		var log_arr = statusResult.getValue('log_entries')
					.QueryInterface(Components.interfaces.nsISupportsArray);
		if(log_arr.Count() > 0) {
		for (var i = 0; i < log_arr.Count(); i++) {
			var newURI = "urn:log." + i;
			var	log_entry;
			var log_type;
			var log_dic = log_arr.QueryElementAt(i, Components.interfaces.nsIDictionary);
			if(log_dic.hasKey('INFO')) {
				log_entry = log_dic.getValue('INFO').
					QueryInterface(Components.interfaces.nsISupportsCString);
				log_type = "info";
			} else if(log_dic.hasKey('WARNING'))  {
				log_entry = log_dic.getValue('WARNING').
					QueryInterface(Components.interfaces.nsISupportsCString);
				log_type = "warning";
			} else if(log_dic.hasKey('ERROR'))  {
				log_entry = log_dic.getValue('ERROR').
					QueryInterface(Components.interfaces.nsISupportsCString);
				log_type = "error";
			}
		 	// Add log data
		 	// log_entry
			ds.Assert(rdf.GetResource(newURI),
				rdf.GetResource("http://home.netscape.com/NC-rdf#logEntry"),
				rdf.GetLiteral(log_entry), true);
			ds.Assert(rdf.GetResource("urn:log"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#log"),
				rdf.GetResource(newURI), true);			
			ds.Assert(rdf.GetResource(newURI),
				rdf.GetResource("http://home.netscape.com/NC-rdf#logType"),
				rdf.GetLiteral(log_type), true);
			ds.Assert(rdf.GetResource("urn:log"),
				rdf.GetResource("http://home.netscape.com/NC-rdf#log"),
				rdf.GetResource(newURI), true);					
			}
		}
		
		// Save RDF to local file (for testing purposes only)
		//saveRDF(ds, "/home/<user>/rdf");
	
		// Rebuild trees
        for (var i = 1; i <= 3; ++i) {
			var tree = document.getElementById("tree" + i);
			if(i == 2) {
				// Rebuild Queue-Tree only if necessary to preserve selection
				var tv = tree.view
							.QueryInterface(Components.interfaces.nsITreeView);
				if(tv.selection.count == 0) {
					tree.builder.rebuild();
				} else {tree.builder.refresh();}
			} else {
				tree.builder.rebuild();
				}
        }
        
		// Check if pause/continue button has the appropriate image
		var paused = statusResult.getValue('is_paused')
				.QueryInterface(Components.interfaces.nsISupportsPRBool);
		var button = document.getElementById('tpc');
		if(paused == 'true') {
			if(button.image == 'img/pause.png') {
				button.setAttribute('image', 'img/continue.png');}
		} else {
			if(button.image == 'img/continue.png') {
				button.setAttribute('image', 'img/pause.png');
				}
			}
	},
		
	onFault: function (client, ctxt, fault) {
		// Stop loop on fault
		window.clearInterval(loop);
		alert('XML-RPC Fault: '+fault);
	},

	onError: function (client, ctxt, status, errorMsg) {
		// Stop loop on error
		window.clearInterval(loop);
		alert('Error: '+errorMsg);
	}
};

//
// FILE
//
// Save RDF datasource 
// Stores in-memory-datasource to local file - for testing purposes only!
function saveRDF(datasource, path) {

	// Init file
	var file = Components.classes["@mozilla.org/file/local;1"]
				.createInstance(Components.interfaces.nsILocalFile);
	file.initWithPath(path);
	
	// Create one if necessary
	if ( file.exists() == false ) {
		file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420 );
	}
	
	// Init stream
	var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
					.createInstance( Components.interfaces.nsIFileOutputStream );
	outputStream.init( file, 0x04 | 0x08 | 0x20, 420, 0 );
	
	// Init serializer
	var serializer=Components.classes["@mozilla.org/rdf/xml-serializer;1"]
					.createInstance(Components.interfaces.nsIRDFXMLSerializer);
	serializer.init(datasource);
	serializer.QueryInterface(Components.interfaces.nsIRDFXMLSource);
	
	// Write to file
	serializer.Serialize(outputStream);
	outputStream.close();
}