/** 
* Copyright (c) 2007 Guido Angenendt <guido.angenendt@stud.uni-due.de>
*
*    This file is part of hellaKommanda.
*
*    hellaKommanda is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    hellaKommanda is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with hellaKommanda.  If not, see <http://www.gnu.org/licenses/>.
**/

//
// Functions
//

// Get extension's path
function getContentPath() {
	var id = "hellakommanda@guidos-corner.de";
	var path = ( Components.classes["@mozilla.org/extensions/manager;1"]
				.getService(Components.interfaces.nsIExtensionManager)
				.getInstallLocation(id)
				.getItemLocation(id)).path;
	
	// determine the file-separator
	if (path.search(/\\/) != -1) {
		path = path + "\\chrome\\hellakommanda\\content\\";
	} else {
		path = path + "/chrome/hellakommanda/content/";
	}
	
	return path;
}

// returns the linebreak for the system
function getLinebreak() {
        if (/win/i.test(navigator.platform))
            return '\r\n';
        else if (/mac/i.test(navigator.platform))
            return '\r';
        else
            return '\n';        
}


// Create sample settings from dummy file
function createSampleSettings() {
	
	// Create component for file writing
	var file = Components.classes["@mozilla.org/file/local;1"]
				.createInstance(Components.interfaces.nsILocalFile);
	var path = getContentPath() + "settings.xml";
    
    // Used to check whether or not you have the proper permissions to 
    // write a file to the local machine	
	try {netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");}
	catch (e) { alert("Permission to write to file was denied.");}
	
	file.initWithPath(path);
	
	// Create file output stream and use write/create/truncate mode
	// 0x02 writing, 0x08 create file, 0x20 truncate length if exist
	var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
				.createInstance( Components.interfaces.nsIFileOutputStream );
	outputStream.init( file, 0x02 | 0x08 | 0x20, 0666, 0);
	
	var xml = '<?xml version="1.0" encoding="UTF-8"?>' + this.getLinebreak();
	xml += '<settings>' + this.getLinebreak();
	xml += '<server>localhost</server>' + this.getLinebreak();
	xml += '<port>8760</port>' + this.getLinebreak();
	xml += '<password>changeme</password>' + this.getLinebreak();
	xml += '</settings>' + this.getLinebreak();	
	
	outputStream.write(xml, xml.length);
	
	outputStream.close();
	
}

// Show/Hide password
function togglePasswd() {

	var checkbox = document.getElementById('showpasswd');
	var textbox = document.getElementById('passwd');
	if (checkbox.checked == false) {
		textbox.type = null;
		} else {
			textbox.type="password";
			}
}

// Save XML document
function saveXML(xmlDoc) {
	
	// Create component for file writing
	var file = Components.classes["@mozilla.org/file/local;1"]
				.createInstance(Components.interfaces.nsILocalFile);
	var path = getContentPath() + "settings.xml";
	file.initWithPath( path );
	
	// Create file output stream and use write/create/truncate mode
	// 0x02 writing, 0x08 create file, 0x20 truncate length if exist
	var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
				.createInstance( Components.interfaces.nsIFileOutputStream );
	outputStream.init( file, 0x02 | 0x08 | 0x20, 0666, 0);
		
	// Write to file
	var stream = new XMLSerializer().serializeToStream(xmlDoc, outputStream, "");
	outputStream.close();
}

// Load values from XML document
function load() {
	
	// Load XML document from disk
	var xmlDoc = document.implementation.createDocument("","",null);
	xmlDoc.async = false;
	try { xmlDoc.load("chrome://hellakommanda/content/settings.xml");}
	catch (e) { 
		if (e.name == "NS_ERROR_FILE_NOT_FOUND") {
			createSampleSettings();
			load();
		};
	}
	
	// Get stored values from document
	var server = xmlDoc.getElementsByTagName("server").item(0).firstChild.nodeValue;
	var port = xmlDoc.getElementsByTagName("port").item(0).firstChild.nodeValue;
	var password = xmlDoc.getElementsByTagName("password").item(0).firstChild.nodeValue;
	
	// Set values in presentation accordingly
	document.getElementById('server').value = server;
	document.getElementById('port').value = port;
	document.getElementById('passwd').value = password;
}

// Save values to XML document
function save() {
	
	// Load XML document from disk
	var xmlDoc = document.implementation.createDocument("","",null);
	xmlDoc.async = false;
	xmlDoc.load("chrome://hellakommanda/content/settings.xml");

	// Get new values
	var server = document.getElementById('server').value;
	var port = document.getElementById('port').value;
	var password = document.getElementById('passwd').value;
	
	// Change values in XML document
	xmlDoc.getElementsByTagName("server").item(0).firstChild.nodeValue = server;
	xmlDoc.getElementsByTagName("port").item(0).firstChild.nodeValue = port;
	xmlDoc.getElementsByTagName("password").item(0).firstChild.nodeValue = password;
	
	// Save document
	saveXML(xmlDoc);
	
	// Close window
	window.close();
}